@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2017 Tobias Stoeckmann <tobias@stoeckmann.org>
+ * Copyright (c) 2017-2018 Tobias Stoeckmann <tobias@stoeckmann.org>
  *
  * Permission to use, copy, modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -66,6 +66,12 @@ typedef struct wp_option {
 	int		 screen;
 } wp_option_t;
 
+typedef struct wp_config {
+	wp_option_t	*options;
+	size_t		 count;
+	int		 daemon;
+} wp_config_t;
+
 typedef struct wp_output {
 	char *name;
 	int16_t x, y;
@@ -73,14 +79,16 @@ typedef struct wp_output {
 } wp_output_t;
 
 extern int	 has_randr;
+extern int	 show_debug;
 
+void		 debug(const char *, ...);
 void		 free_outputs(wp_output_t *);
 wp_output_t	*get_output(wp_output_t *, char *);
 wp_output_t	*get_outputs(xcb_connection_t *, xcb_screen_t *);
 pixman_image_t	*load_jpeg(FILE *);
 pixman_image_t	*load_png(FILE *);
 pixman_image_t	*load_xpm(xcb_connection_t *, xcb_screen_t *, FILE *);
-wp_option_t	*parse_options(char **);
+wp_config_t	*parse_config(char **);
 void		 stage1_sandbox(void);
 void		 stage2_sandbox(void);
 void		*xmalloc(size_t);
