@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2017 Tobias Stoeckmann <tobias@stoeckmann.org>
+ * Copyright (c) 2017-2018 Tobias Stoeckmann <tobias@stoeckmann.org>
  *
  * Permission to use, copy, modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -14,6 +14,11 @@
  * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
 
+#include "config.h"
+
+#ifdef WITH_RANDR
+  #include <xcb/randr.h>
+#endif /* WITH_RANDR */
 #include <xcb/xcb.h>
 #include <xcb/xcb_image.h>
 
@@ -24,7 +29,6 @@
 #include <string.h>
 #include <unistd.h>
 
-#include "config.h"
 #include "functions.h"
 
 #define ATOM_ESETROOT "ESETROOT_PMAP_ID"
@@ -45,7 +49,7 @@ get_max_rows_per_request(xcb_connection_t *c, xcb_image_t *image, uint32_t n)
 	max_height = max_len / row_len;
 	if (max_height < 1)
 		errx(1, "unable to put image on X server");
-	DBG("put image request parameters:\n"
+	debug("put image request parameters:\n"
 	    "maximum request length allowed for server (32 bits): %u\n"
 	    "maximum length for row data: %u\n"
 	    "length of rows in image: %u\n"
@@ -92,7 +96,7 @@ load_pixman_images(xcb_connection_t *c, xcb_screen_t *screen,
 
 	for (option = options; option->filename != NULL; option++)
 		if (option->buffer->pixman_image == NULL) {
-			DBG("loading %s\n", option->filename);
+			debug("loading %s\n", option->filename);
 			img = load_pixman_image(c, screen, option->buffer->fp);
 			if (img == NULL)
 				errx(1, "failed to parse %s", option->filename);
@@ -141,7 +145,7 @@ tile(pixman_image_t *dest, wp_output_t *output, wp_option_t *option)
 			else
 				w = pixman_width;
 
-			DBG("tiling %s for %s (area %dx%d+%d+%d)\n",
+			debug("tiling %s for %s (area %dx%d+%d+%d)\n",
 			    option->filename, output->name != NULL ?
 			    output->name : "screen", w, h, off_x, off_y);
 			pixman_image_composite(PIXMAN_OP_CONJOINT_SRC,
@@ -203,7 +207,7 @@ transform(pixman_image_t *dest, wp_output_t *output, wp_option_t *option)
 	pixman_transform_from_pixman_f_transform(&transform, &ftransform);
 	pixman_image_set_transform(pixman_image, &transform);
 
-	DBG("composing %s for %s (area %dx%d+%d+%d) (mode %d)\n",
+	debug("composing %s for %s (area %dx%d+%d+%d) (mode %d)\n",
 	    option->filename, output->name != NULL ? output->name : "screen",
 	    output->width, output->height, 0, 0, option->mode);
 	pixman_image_composite(PIXMAN_OP_CONJOINT_SRC, pixman_image, NULL, dest,
@@ -218,14 +222,14 @@ put_wallpaper(xcb_connection_t *c, xcb_screen_t *screen, wp_output_t *output,
 	uint32_t h, max_height, row_len, sub_height;
 	xcb_image_t *sub;
 
-	DBG("xcb image (%dx%d) to %s (%dx%d+%d+%d)\n",
+	debug("xcb image (%dx%d) to %s (%dx%d+%d+%d)\n",
 	    xcb_image->width, xcb_image->height,
 	    output->name != NULL ? output->name : "screen", output->width,
 	    output->height, output->x, output->y);
 
 	max_height = get_max_rows_per_request(c, xcb_image, UINT32_MAX / 4);
 	if (max_height < xcb_image->height) {
-		DBG("image exceeds request size limitations\n");
+		debug("image exceeds request size limitations\n");
 
 		/* adjust for better performance */
 		max_height = get_max_rows_per_request(c, xcb_image, 65536);
@@ -255,7 +259,7 @@ put_wallpaper(xcb_connection_t *c, xcb_screen_t *screen, wp_output_t *output,
 				errx(1, "failed to create xcb image");
 		}
 
-		DBG("put image (%dx%d+0+%d) to %s (%dx%d+%d+%d)\n",
+		debug("put image (%dx%d+0+%d) to %s (%dx%d+%d+%d)\n",
 		    sub->width, sub->height, h,
 		    output->name != NULL ? output->name : "screen",
 		    sub->width, sub_height, output->x,
@@ -310,6 +314,7 @@ process_output(xcb_connection_t *c, xcb_screen_t *screen, wp_output_t *output,
 static void
 update_atoms(xcb_connection_t *c, xcb_screen_t *screen, xcb_pixmap_t pixmap)
 {
+	static int first_run = 1;
 	int i;
 	xcb_intern_atom_cookie_t atom_cookie[2];
 	xcb_intern_atom_reply_t *atom_reply[2];
@@ -344,10 +349,13 @@ update_atoms(xcb_connection_t *c, xcb_screen_t *screen, xcb_pixmap_t pixmap)
 			old[i] = NULL;
 	}
 
-	if (old[0] != NULL)
-		xcb_kill_client(c, *old[0]);
-	if (old[1] != NULL && (old[0] == NULL || *old[0] != *old[1]))
-		xcb_kill_client(c, *old[1]);
+	if (first_run) {
+		if (old[0] != NULL)
+			xcb_kill_client(c, *old[0]);
+		if (old[1] != NULL && (old[0] == NULL || *old[0] != *old[1]))
+			xcb_kill_client(c, *old[1]);
+		first_run = 0;
+	}
 
 	for (i = 0; i < 2; i++) {
 		if (atom_reply[i] != NULL)
@@ -392,7 +400,7 @@ process_screen(xcb_connection_t *c, xcb_screen_t *screen, int snum,
 		outputs = get_outputs(c, screen);
 	}
 
-	DBG("creating pixmap (%dx%d)\n", width, height);
+	debug("creating pixmap (%dx%d)\n", width, height);
 	pixmap = xcb_generate_id(c);
 	xcb_create_pixmap(c, screen->root_depth, pixmap, screen->root, width,
 	    height);
@@ -437,29 +445,63 @@ static void
 usage(void)
 {
 	fprintf(stderr,
-"usage: xwallpaper [--screen <screen>] [--no-randr] [--output <output>]\n"
-"  [--center <file>] [--maximize <file>]  [--stretch <file>]\n"
-"  [--tile <file>] [--zoom <file>] [--version]\n");
+"usage: xwallpaper [--screen <screen>] [--daemon] [--no-randr]\n"
+"  [--output <output>] [--center <file>] [--maximize <file>]\n"
+"  [--stretch <file>] [--tile <file>] [--zoom <file>] [--version]\n");
 	exit(1);
 }
 
+#ifdef WITH_RANDR
+static void
+process_event(wp_config_t *config, xcb_connection_t *c,
+    xcb_generic_event_t *event) {
+	xcb_randr_screen_change_notify_event_t *randr_event;
+	xcb_screen_iterator_t it;
+	int snum;
+
+	randr_event = (xcb_randr_screen_change_notify_event_t *)event;
+	debug("event received: response_type=%u, sequence=%u\n",
+	    randr_event->response_type, randr_event->sequence);
+	it = xcb_setup_roots_iterator(xcb_get_setup(c));
+	for (snum = 0; it.rem; snum++, xcb_screen_next(&it)) {
+		if (it.data->root == randr_event->root) {
+			/* only redraw if really necessary */
+			if (it.data->width_in_pixels == randr_event->width &&
+			    it.data->height_in_pixels == randr_event->height)
+				break;
+			it.data->width_in_pixels = randr_event->width;
+			it.data->height_in_pixels = randr_event->height;
+			process_screen(c, it.data, snum, config->options);
+		}
+	}
+	xcb_request_check(c, xcb_set_close_down_mode(c,
+	    XCB_CLOSE_DOWN_RETAIN_PERMANENT));
+	if (xcb_connection_has_error(c))
+		warnx("error encountered while setting wallpaper");
+}
+#endif /* WITH_RANDR */
+
 int
 main(int argc, char *argv[])
 {
-	wp_option_t *options;
+	wp_config_t *config;
 	xcb_connection_t *c;
+	xcb_generic_event_t *event;
 	xcb_screen_iterator_t it;
 	int snum;
 #ifdef HAVE_PLEDGE
-	if (pledge("dns inet rpath stdio unix", NULL) == -1)
+	if (pledge("dns inet proc rpath stdio unix", NULL) == -1)
 		err(1, "pledge");
 #endif /* HAVE_PLEDGE */
 #ifdef WITH_SECCOMP
 	stage1_sandbox();
 #endif /* WITH_SECCOMP */
-	if (argc < 2 || (options = parse_options(++argv)) == NULL)
+	if (argc < 2 || (config = parse_config(++argv)) == NULL)
 		usage();
 
+	if (config->daemon && daemon(0, show_debug) < 0)
+		warnx("failed to daemonize");
+
 	c = xcb_connect(NULL, NULL);
 	if (xcb_connection_has_error(c))
 		errx(1, "failed to connect to X server");
@@ -481,16 +523,28 @@ main(int argc, char *argv[])
 	 */
 	if (it.rem == 0)
 		errx(1, "no screen found");
-	load_pixman_images(c, it.data, options);
+	load_pixman_images(c, it.data, config->options);
 
 	for (snum = 0; it.rem; snum++, xcb_screen_next(&it))
-		process_screen(c, it.data, snum, options);
+		process_screen(c, it.data, snum, config->options);
 
 	xcb_request_check(c,
 	    xcb_set_close_down_mode(c, XCB_CLOSE_DOWN_RETAIN_PERMANENT));
 	if (xcb_connection_has_error(c))
 		warnx("error encountered while setting wallpaper");
 
+#ifdef WITH_RANDR
+	if (config->daemon) {
+		it = xcb_setup_roots_iterator(xcb_get_setup(c));
+		for (snum = 0; it.rem; snum++, xcb_screen_next(&it))
+			xcb_request_check(c, xcb_randr_select_input(c, it.data->root,
+			    XCB_RANDR_NOTIFY_MASK_SCREEN_CHANGE));
+
+		while ((event = xcb_wait_for_event(c)) != NULL)
+			process_event(config, c, event);
+	}
+#endif /* WITH_RANDR */
+
 	xcb_disconnect(c);
 
 	return 0;
